/* -*-c-*- */
/* Complex base type.  */
#define T_FFI_TYPE ffi_type_complex_longdouble
/* C type corresponding to the base type.  */
#define T_C_TYPE long double
/* C cast for a value of type T_C_TYPE that is passed to printf.  */
#define T_CONV
